@@ -10,12 +10,12 @@
 #'   rewritten based on a new search for Scala and Java.  If \code{FALSE}, the
 #'   previous configuration is sourced from the script
 #'   \code{~/.rscala/config.R}.  If \code{"live"}, a new search is performed,
-#'   but the results do not overwrite the previous configuration script.  Finally,
-#'   the value set here is superceded by the value of the environment variable
-#'   \code{RSCALA_RECONFIG}, if it exists.
+#'   but the results do not overwrite the previous configuration script.
+#'   Finally, the value set here is superceded by the value of the environment
+#'   variable \code{RSCALA_RECONFIG}, if it exists.
 #' @param download A character vector which may be length-zero or whose elements
 #'   are any combination of \code{"java"}, \code{"scala"}, or \code{"sbt"}. Or,
-#'   \code{TRUE} denotes all three.  The indicated software will be installed at
+#'   \code{TRUE} denotes all three.  The indicated software will be installed in
 #'   "~/.rscala".
 #' @param require.sbt Should SBT be required, downloading and installing it in
 #'   '~/.rscala/sbt' if necessary?
@@ -37,7 +37,7 @@ scalaConfig <- function(verbose=TRUE, reconfig=FALSE, download=character(0), req
   download.sbt <- "sbt" %in% download
   if ( Sys.getenv("RSCALA_RECONFIG") != "" ) reconfig <- Sys.getenv("RSCALA_RECONFIG")
   consent <- identical(reconfig,TRUE) || download.java || download.scala || download.sbt
-  installPath <- path.expand(file.path("~",".rscala"))
+  installPath <- path.expand(if ( Sys.getenv("RSCALA_HOME") != "" ) Sys.getenv("RSCALA_HOME") else file.path("~",".rscala"))
   dependsPath <- if ( Sys.getenv("RSCALA_BUILDING") != "" ) file.path(getwd(),"inst","dependencies") else ""
   offerInstall <- function(msg) {
     if ( !identical(reconfig,"live") && interactive() ) {
@@ -49,7 +49,7 @@ scalaConfig <- function(verbose=TRUE, reconfig=FALSE, download=character(0), req
       }
     } else FALSE
   }
-  configPath  <- file.path(installPath,"config.R")
+  configPath <- file.path(installPath,"config.R")
   if ( identical(reconfig,FALSE) && file.exists(configPath) && !download.java && !download.scala && !download.sbt ) {
     if ( verbose ) cat(paste0("\nRead existing configuration file: ",configPath,"\n\n"))
     source(configPath,chdir=TRUE,local=TRUE)
@@ -94,9 +94,14 @@ scalaConfig <- function(verbose=TRUE, reconfig=FALSE, download=character(0), req
         if ( dependsPath != "" ) {
           installSoftware(dependsPath,"scala",verbose=verbose)
           scalaConf <- findExecutable("scala","Scala",dependsPath,scalaSpecifics2,verbose)
-          if ( is.null(scalaConf) ) stop(stopMsg)
         }
-        else stop(stopMsg)
+        if ( is.null(scalaConf) ) {
+          tmpdir <- tempdir()
+          installSoftware(tmpdir,"scala",verbose=verbose)
+          scalaConf <- findExecutable("scala","Scala",tmpdir,scalaSpecifics2,verbose)
+          if ( is.null(scalaConf) ) stop(stopMsg)
+          else if ( verbose || interactive() ) cat(stopMsg)
+        }
       }
       consent <- consent || consent2
     }
@@ -111,7 +116,7 @@ scalaConfig <- function(verbose=TRUE, reconfig=FALSE, download=character(0), req
       if ( consent2 ) {
         installSoftware(installPath,"sbt",verbose=verbose)
         sbtConf <- findExecutable("sbt","SBT",installPath,sbtSpecifics,verbose)
-        if ( is.null(sbtConf) )  stop(stopMsg)
+        if ( is.null(sbtConf) ) stop(stopMsg)
       } else stop(stopMsg)
       consent <- consent || consent2      
     }
@@ -146,7 +151,7 @@ findExecutable <- function(mode,prettyMode,installPath,mapper,verbose=TRUE) {  #
   allCaps <- toupper(mode)
   if ( verbose ) cat(paste0("\nSearching the system for ",prettyMode,".\n"))
   ###
-  label <- "directory"
+  label <- "rscala home directory"
   regex <- sprintf("%s%s$",mode,if ( .Platform$OS.type == "windows" ) "(\\.exe|\\.bat)" else "")
   candidates <- list.files(installPath,paste0("^",regex),recursive=TRUE)
   candidates <- candidates[grepl(sprintf("^%s/(.*/|)bin/%s",mode,regex),candidates)]
@@ -169,20 +174,21 @@ findExecutable <- function(mode,prettyMode,installPath,mapper,verbose=TRUE) {  #
     conf <- tryCandidate(if ( ! inherits(home,"try-error") && ( home != "" ) ) file.path(home,"bin",mode) else "")
     if ( ! is.null(conf) ) return(conf)
   } else {
+    ###
     label <- "PATH environment variable"
     conf <- tryCandidate(Sys.which(mode)[[mode]])
     if ( ! is.null(conf) ) return(conf)
+    ###
+    if ( mode == "java" ) {
+      label <- paste0("R CMD config JAVA")
+      path <- tryCatch(system2(file.path(R.home("bin"),"R"),c("CMD","config","JAVA"),stdout=TRUE,stderr=FALSE), warning=function(e) NULL, error=function(e) NULL)
+      conf <- tryCandidate(path)
+      if ( ! is.null(conf) ) return(conf)
+    }
   }
   ###
-  if ( mode == "java" ) {
-    label <- paste0("R CMD config JAVA")
-    path <- tryCatch(system2(file.path(R.home("bin"),"R"),c("CMD","config","JAVA"),stdout=TRUE,stderr=FALSE), warning=function(e) NULL, error=function(e) NULL)
-    conf <- tryCandidate(path)
-    if ( ! is.null(conf) ) return(conf)
-  } 
-  ###
   if ( Sys.getenv("RSCALA_BUILDING") == "" ) {
-    label <- "package build directory"
+    label <- "rscala package build directory"
     regex <- sprintf("%s%s$",mode,if ( .Platform$OS.type == "windows" ) "(\\.exe|\\.bat)" else "")
     dependsPath <- file.path(system.file(package="rscala"),"dependencies")
     candidates <- list.files(dependsPath,paste0("^",regex),recursive=TRUE)
@@ -224,11 +230,11 @@ installSoftware <- function(installPath, software, version, os, arch, verbose=FA
   sel <- urls$software == software
   if ( missing(version) ) {
     version <- if ( software == "java" ) {
-      if ( Sys.getenv("RSCALA_VERIFY_JAVA_VERSION","") != "" )  Sys.getenv("RSCALA_VERIFY_JAVA_VERSION","") else "8"
+      if ( Sys.getenv("RSCALA_JAVA_VERSION","") != "" )  Sys.getenv("RSCALA_JAVA_VERSION","") else "8"
     } else if ( software == "scala" ) {
-      if ( Sys.getenv("RSCALA_VERIFY_SCALA_VERSION","") != "" ) Sys.getenv("RSCALA_VERIFY_SCALA_VERSION","") else "2.12"
+      if ( Sys.getenv("RSCALA_SCALA_VERSION","") != "" ) Sys.getenv("RSCALA_SCALA_VERSION","") else "2.12"
     } else if ( software == "sbt" ) {
-      if ( Sys.getenv("RSCALA_VERIFY_SBT_VERSION","") != "" ) Sys.getenv("RSCALA_VERIFY_SBT_VERSION","") else "1.2"
+      if ( Sys.getenv("RSCALA_SBT_VERSION","") != "" ) Sys.getenv("RSCALA_SBT_VERSION","") else "1.2"
     } else NULL
   }
   sel <- sel & (urls$version == version)
@@ -238,7 +244,7 @@ installSoftware <- function(installPath, software, version, os, arch, verbose=FA
   sel <- sel & (urls$os == os)
   if ( missing(arch) ) {
     arch <- if ( software == "java" ) {
-      if ( Sys.getenv("RSCALA_VERIFY_JAVA_ARCH","") != "" ) Sys.getenv("RSCALA_VERIFY_JAVA_ARCH","") else R.Version()$arch
+      if ( Sys.getenv("RSCALA_JAVA_ARCH","") != "" ) Sys.getenv("RSCALA_JAVA_ARCH","") else R.Version()$arch
     } else "any"
   }
   sel <- if ( arch == "any" ) sel else {
@@ -251,8 +257,8 @@ installSoftware <- function(installPath, software, version, os, arch, verbose=FA
     if ( len == 1 ) cat(paste0("There is 1 candidate.\n\n"))
     else cat(paste0("There are ",len," candidates.\n\n"))
   }
-  if ( missing(downloadFailureCount) && ( Sys.getenv("RSCALA_VERIFY_DOWNLOAD_FAILURE_COUNT","") != "" ) ) downloadFailureCount <- as.integer(Sys.getenv("RSCALA_VERIFY_DOWNLOAD_FAILURE_COUNT",""))
-  if ( missing(extractFailureCount)  && ( Sys.getenv("RSCALA_VERIFY_EXTRACT_FAILURE_COUNT", "") != "" ) ) extractFailureCount  <- as.integer(Sys.getenv("RSCALA_VERIFY_EXTRACT_FAILURE_COUNT", ""))
+  if ( missing(downloadFailureCount) && ( Sys.getenv("RSCALA_DOWNLOAD_FAILURE_COUNT","") != "" ) ) downloadFailureCount <- as.integer(Sys.getenv("RSCALA_DOWNLOAD_FAILURE_COUNT",""))
+  if ( missing(extractFailureCount)  && ( Sys.getenv("RSCALA_EXTRACT_FAILURE_COUNT", "") != "" ) ) extractFailureCount  <- as.integer(Sys.getenv("RSCALA_EXTRACT_FAILURE_COUNT", ""))
   dfc <- efc <- 0
   for ( candidate in candidates ) {
     archivePath <- file.path(tempdir(), basename(candidate))
@@ -299,7 +305,7 @@ setJavaEnv <- function(javaConf) {
   if ( oldJAVACMD  == "--unset--" ) oldJAVACMD  <- NULL
   if ( oldJAVAHOME == "--unset--" ) oldJAVAHOME <- NULL
   if ( is.null(javaConf$javaCmd)  ) Sys.unsetenv("JAVACMD")   else Sys.setenv(JAVACMD= javaConf$javaCmd)
-  if ( is.null(javaConf$javaHome) ) Sys.unsetenv("JAVA_HOME") else Sys.setenv(JAVAHOME=javaConf$javaHome)
+  if ( is.null(javaConf$javaHome) ) Sys.unsetenv("JAVA_HOME") else Sys.setenv(JAVA_HOME=javaConf$javaHome)
   list(javaCmd=oldJAVACMD,javaHome=oldJAVAHOME)
 }
 
@@ -313,11 +319,15 @@ scalaSpecifics <- function(scalaCmd,javaConf,verbose) {
   oldJavaEnv <- setJavaEnv(javaConf)
   info <- tryCatch({
     system2(scalaCmd,c("-nobootcp","-nc","-e",shQuote('import util.Properties._; println(Seq(versionNumberString,scalaHome,javaHome).mkString(lineSeparator))')),stdout=TRUE,stderr=FALSE)
-   }, warning=function(e) "")
+  }, warning=function(e) "", error=function(e) "")
   setJavaEnv(oldJavaEnv)
-  fullVersion <- info[1]
-  majorVersion <- scalaMajorVersion(fullVersion)
-  if ( majorVersion == "" ) majorVersion <- "?"
+  if ( is.null(info[1]) || is.na(info[1]) ) {
+    fullVersion <- "?"
+    majorVersion <- "?"
+  } else {
+    fullVersion <- info[1]
+    majorVersion <- scalaMajorVersion(fullVersion)
+  }
   supportedVersions <- names(scalaVersionJARs())
   if ( ( length(supportedVersions) > 0 ) && ! ( majorVersion %in% supportedVersions ) ) paste0("unsupported Scala version: ",majorVersion)
   else {
@@ -333,9 +343,9 @@ verifyDownloads <- function() {
   scalaConfig(download=c("java","scala","sbt"))
   for ( version in c("11","8") ) {
     for ( efc in 0:0 ) {
-      Sys.setenv(RSCALA_VERIFY_EXTRACT_FAILURE_COUNT=efc)
-      Sys.setenv(RSCALA_VERIFY_JAVA_VERSION=version)
-      Sys.setenv(RSCALA_VERIFY_JAVA_ARCH="i386")
+      Sys.setenv(RSCALA_EXTRACT_FAILURE_COUNT=efc)
+      Sys.setenv(RSCALA_JAVA_VERSION=version)
+      Sys.setenv(RSCALA_JAVA_ARCH="i386")
       cat(paste0("----------\nefc=",efc,", software='java', version=",version,", arch=i386\n"))
       scalaConfig(download="java")
       s <- scala()
@@ -345,9 +355,9 @@ verifyDownloads <- function() {
   }
   for ( version in c("11","8") ) {
     for ( efc in 0:2 ) {
-      Sys.setenv(RSCALA_VERIFY_EXTRACT_FAILURE_COUNT=efc)
-      Sys.setenv(RSCALA_VERIFY_JAVA_VERSION=version)
-      Sys.setenv(RSCALA_VERIFY_JAVA_ARCH="x86_64")
+      Sys.setenv(RSCALA_EXTRACT_FAILURE_COUNT=efc)
+      Sys.setenv(RSCALA_JAVA_VERSION=version)
+      Sys.setenv(RSCALA_JAVA_ARCH="x86_64")
       cat(paste0("----------\nefc=",efc,", software='java', version=",version,"\n"))
       scalaConfig(download="java")
       s <- scala()
@@ -355,10 +365,10 @@ verifyDownloads <- function() {
       close(s)
     }
   }
-  for ( version in c("2.11","2.13.0-RC2","2.12") ) {
+  for ( version in c("2.11","2.12","2.13") ) {
     for ( efc in 0:1 ) {
-      Sys.setenv(RSCALA_VERIFY_EXTRACT_FAILURE_COUNT=efc)
-      Sys.setenv(RSCALA_VERIFY_SCALA_VERSION=version)
+      Sys.setenv(RSCALA_EXTRACT_FAILURE_COUNT=efc)
+      Sys.setenv(RSCALA_SCALA_VERSION=version)
       cat(paste0("----------\nefc=",efc,", software='scala', version=",version,"\n"))
       scalaConfig(download="scala")
       s <- scala()
@@ -368,13 +378,13 @@ verifyDownloads <- function() {
   }
   for ( version in c("1.2") ) {
     for ( efc in 0:1 ) {
-      Sys.setenv(RSCALA_VERIFY_EXTRACT_FAILURE_COUNT=efc)
-      Sys.setenv(RSCALA_VERIFY_SBT_VERSION=version)
+      Sys.setenv(RSCALA_EXTRACT_FAILURE_COUNT=efc)
+      Sys.setenv(RSCALA_SBT_VERSION=version)
       cat(paste0("----------\nefc=",efc,", software='sbt', version=",version,"\n"))
       scalaConfig(download="sbt")
       scalaSBT("sbtVersion")
       cat("\n")
     }
   }
-  Sys.unsetenv(c("RSCALA_VERIFY_EXTRACT_FAILURE_COUNT","RSCALA_VERIFY_JAVA_VERSION","RSCALA_VERIFY_SCALA_VERSION","RSCALA_VERIFY_SBT_VERSION","RSCALA_VERIFY_JAVA_ARCH"))
+  Sys.unsetenv(c("RSCALA_EXTRACT_FAILURE_COUNT","RSCALA_JAVA_VERSION","RSCALA_SCALA_VERSION","RSCALA_SBT_VERSION","RSCALA_JAVA_ARCH"))
 }
